-- Demonstration script for the presentation "Improving SQL Server Performance by Writing Faster T-SQL" by Andy Novick
-- Copyright (C) 2012-2014 by Andrew Novick,
-- This script is intended for demonstration purposes only. Do not use this in production situations.
USE tempdb
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
SET ANSI_PADDING ON
go

IF NOT EXISTS (SELECT * from sys.synonyms where [name]='aw_sales_SalesOrderDetail')
	CREATE SYNONYM aw_sales_SalesOrderDetail FOR AdventureWorks2014.Sales.SalesOrderDetail
IF NOT EXISTS (SELECT * from sys.synonyms where [name]='aw_sales_SalesOrderHeader')
	CREATE SYNONYM aw_sales_SalesOrderHeader FOR AdventureWorks2014.Sales.SalesOrderHeader
IF NOT EXISTS (SELECT * from sys.synonyms where [name]='aw_sales_SalesReason')
	CREATE SYNONYM aw_sales_SalesReason FOR AdventureWorks2014.Sales.SalesReason
IF NOT EXISTS (SELECT * from sys.synonyms where [name]='aw_sales_SalesOrderHeaderSalesReason')
	CREATE SYNONYM aw_sales_SalesOrderHeaderSalesReason FOR AdventureWorks2014.Sales.SalesOrderHeaderSalesReason
IF NOT EXISTS (SELECT * from sys.synonyms where [name]='aw_sales_Purchasing_ShipMethod')
	CREATE SYNONYM aw_sales_Purchasing_ShipMethod FOR AdventureWorks2014.Purchasing.ShipMethod

go

IF OBJECT_ID('dbo.ns_sample_level_2', 'P') IS NOT NULL DROP PROC dbo.ns_sample_level_2
go

CREATE PROC dbo.ns_sample_level_2
	
	@parm1 INT = 0
   ,@num_sales_rows INT output

AS BEGIN;

	WAITFOR DELAY '00:00:00.015'

END;
go









IF OBJECT_ID('dbo.ns_sample_level_1_a', 'P') IS NOT NULL DROP PROC dbo.ns_sample_level_1_a
go

CREATE PROC dbo.ns_sample_level_1_a
	
	@parm1 INT = 0
   ,@num_sales_rows INT output

AS BEGIN;

	IF OBJECT_ID('tempdb..#sales') IS NOT NULL DROP TABLE #sales

	CREATE TABLE #sales(
		[RevisionNumber] [tinyint] NOT NULL,
		[OrderDate] [datetime] NOT NULL,
		[DueDate] [datetime] NOT NULL,
		[SalesReasonID] INT NULL,
		[ShipMethod] [nvarchar](50) NULL,
		[SalesOrderID] [int] NOT NULL,
		[SalesOrderDetailID] [int] NOT NULL,
		[CarrierTrackingNumber] [nvarchar](25) NULL,
		[OrderQty] [smallint] NOT NULL,
		[ProductID] [int] NOT NULL,
		[SpecialOfferID] [int] NOT NULL,
		[UnitPrice] [money] NOT NULL,
		[UnitPriceDiscount] [money] NOT NULL,
		[LineTotal] [numeric](38, 6) NOT NULL,
		[rowguid] [uniqueidentifier] NOT NULL,
		[ModifiedDate] [datetime] NOT NULL)
	CREATE CLUSTERED INDEX ci_#sales ON #sales (SalesOrderID, SalesOrderDetailID, RevisionNumber);
	CREATE INDEX ix_#sales_reason ON #sales (SalesReasonID, ShipMethod, ProductID, OrderQty)
	--CREATE STATISTICS stat_#sales_shipmethod on #sales (Shipmethod) WITH NORECOMPUTE;
	--CREATE STATISTICS stat_#sales_shipmethod on #sales (ProductID) WITH NORECOMPUTE;

	INSERT INTO #sales   ( RevisionNumber,OrderDate,DueDate, SalesReasonID, ShipMethod,SalesOrderID,SalesOrderDetailID,
	          CarrierTrackingNumber,OrderQty ,         ProductID ,	          SpecialOfferID ,	          UnitPrice ,	          UnitPriceDiscount ,	          LineTotal ,	          rowguid ,	          ModifiedDate	        )
	SELECT 	soh.RevisionNumber, Soh.OrderDate, soh.DueDate, sohsr.SalesReasonID
	    , sm.Name ShipMethod
		, sod.*		
		FROM aw_sales_salesorderheader soh
			INNER join aw_sales_SalesOrderDetail sod
				ON soh.SalesOrderID = sod.SalesOrderID
			LEFT OUTER JOIN aw_sales_SalesOrderHeaderSalesReason sohsr
			ON soh.SalesOrderID = sohsr.SalesOrderID
			LEFT OUTER JOIN aw_sales_SalesReason sr
				ON sohsr.SalesReasonID = sr.SalesReasonID	
			LEFT OUTER JOIN aw_sales_Purchasing_ShipMethod	sm
				ON soh.ShipMethodID = sm.ShipMethodID		
	SELECT @num_sales_rows = COUNT(*) FROM 	#sales

	EXEC ns_sample_level_2 @parm1, @num_sales_rows output
		
	INSERT INTO #reasons (Reason, ShipMethod, ProductID, total_order_quantity)	
	SELECT sr.Name Reason, ShipMethod, ProductID, SUM(OrderQty) total_order_quantity
		FROM #sales s
		INNER JOIN aw_sales_SalesReason sr
			ON s.SalesReasonID = sr.SalesReasonID
		GROUP BY 	sr.Name, ShipMethod, ProductID
	
	UPDATE #reasons SET Reason = 'Unknown' WHERE reason IS NULL
		
-- Cleanup
DROP TABLE #sales


END;
go
























IF OBJECT_ID('dbo.ns_sample_level_1_b', 'P') IS NOT NULL DROP PROC dbo.ns_sample_level_1_b
go

CREATE PROC dbo.ns_sample_level_1_B
	
	@parm1 INT = 0
   ,@num_sales_rows INT output

AS BEGIN;


	CREATE TABLE #sales(
		[RevisionNumber] [tinyint] NOT NULL,
		[OrderDate] [datetime] NOT NULL,
		[DueDate] [datetime] NOT NULL,
		[Reason] [nvarchar](50) NULL,
		[ShipMethod] [nvarchar](50) NULL,
		[SalesOrderID] [int] NOT NULL,
		[SalesOrderDetailID] [int] NOT NULL,
		[CarrierTrackingNumber] [nvarchar](25) NULL,
		[OrderQty] [smallint] NOT NULL,
		[ProductID] [int] NOT NULL,
		[SpecialOfferID] [int] NOT NULL,
		[UnitPrice] [money] NOT NULL,
		[UnitPriceDiscount] [money] NOT NULL,
		[LineTotal] [numeric](38, 6) NOT NULL,
		[rowguid] [uniqueidentifier] NOT NULL,
		[ModifiedDate] [datetime] NOT NULL)

	INSERT INTO #sales
	        ( RevisionNumber,OrderDate,DueDate, Reason,ShipMethod,SalesOrderID,SalesOrderDetailID,
	          CarrierTrackingNumber,OrderQty ,         ProductID ,	          SpecialOfferID ,	          UnitPrice ,	          UnitPriceDiscount ,LineTotal,rowguid,ModifiedDate)
	SELECT 	soh.RevisionNumber, Soh.OrderDate, soh.DueDate, sr.[Name] Reason, sm.[Name] ShipMethod
		, sod.*		
		FROM aw_sales_salesorderheader soh
			INNER join aw_sales_SalesOrderDetail sod
				ON soh.SalesOrderID = sod.SalesOrderID
			LEFT OUTER JOIN aw_sales_SalesOrderHeaderSalesReason sohsr
				ON soh.SalesOrderID = sohsr.SalesOrderID
			LEFT OUTER JOIN aw_sales_salesReason sr
				ON sohsr.SalesReasonID = sr.SalesReasonID
			LEFT OUTER JOIN aw_sales_Purchasing_ShipMethod	sm
				ON soh.ShipMethodID = sm.ShipMethodID
	SET @num_sales_rows = @@ROWCOUNT			

	SELECT COALESCE(Reason, 'Unknown') Reason, ShipMethod, ProductID, SUM(OrderQty) total_order_quantity
		FROM #sales
		GROUP BY Reason, ShipMethod, ProductID

-- Cleanup
DROP TABLE #sales


END;
go

IF OBJECT_ID('dbo.ns_sample_master_proc') IS NOT NULL DROP PROC dbo.ns_sample_master_proc
go












CREATE PROC dbo.ns_sample_master_proc

	@num_sales INT OUTPUT
	
AS BEGIN;

	DECLARE @num_sales1 INT , @num_sales2 INT;
	
	WAITFOR DELAY '00:00:00.5'

	CREATE TABLE #reasons(
		[Reason] [nvarchar](50) NULL,
		[ShipMethod] [nvarchar](50) NULL,
		[ProductID] [int] NOT NULL,
		[total_order_quantity] [int] NULL)
	CREATE CLUSTERED INDEX ci_#reasos__reason ON #reasons (reason, ShipMethod, ProductID)
	
	INSERT INTO #Reasons (Reason, ShipMethod, ProductID, total_order_quantity)
	EXEC ns_sample_level_1_a 1, @num_Sales output

	INSERT INTO #Reasons (Reason, ShipMethod, ProductID, total_order_quantity)
	EXEC ns_sample_level_1_a 1, @num_Sales output

	INSERT INTO #Reasons (Reason, ShipMethod, ProductID, total_order_quantity)
	EXEC ns_sample_level_1_B 2, @num_sales2 output

	SELECT @num_sales = @num_sales1 + @num_sales2
	
	SELECT Reason, ShipMethod, ProductID,SUM(total_order_quantity) total_qty
		FROM #reasons 
		GROUP BY Reason, ShipMethod, ProductID
		ORDER BY ShipMethod, Reason, ProductID 

END;
go


