-- Demonstration script for the presentation "Improving SQL Server Performance by Writing Faster T-SQL" by Andy Novick
-- Copyright (C) 2012-2014 by Andrew Novick,
-- This script is intended for demonstration purposes only. Do not use this in production situations.

SET STATISTICS TIME OFF 
SET STATISTICS IO OFF 
GO

USE tempdb
go
IF OBJECT_ID('dbo.fn_date_from_datetime') IS NOT NULL DROP FUNCTION dbo.fn_date_from_datetime
go

CREATE FUNCTION dbo.fn_date_from_datetime (
	@dt DATETIME
) RETURNS DATE
AS BEGIN
	RETURN CONVERT(DATE, @dt)   	
END	
go
USE AdventureWorks2014
go
SET STATISTICS TIME ON
go -->>>>>>>>>>>>>>>>>>>>>>> END OF SETUP >>>>>>>>>>>>>>>>>>>
IF OBJECT_ID('tempdpb..#temp_destination') IS NOT NULL DROP TABLE #temp_destination
go

sp_configure 'cost threshold for parallelism', 0;
RECONFIGURE
go -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


IF OBJECT_ID('tempdb..#temp_destination') IS NOT NULL DROP TABLE #temp_destination
go

--- CTRL+M -- Include Actual Execution plan


SELECT sod.*
	 , tempdb.dbo.fn_date_from_datetime(sod.ModifiedDate) ModDate	 
	 INTO #temp_destination
	FROM AdventureWorks2014.sales.SalesOrderDetail sod
	INNER JOIN AdventureWorks2014.Production.Product p ON sod.ProductID = p.ProductID
	ORDER BY Style
GO -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>





GO -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

SET STATISTICS TIME OFF 
GO
USE tempdb
go
IF OBJECT_ID('dbo.fn_date_from_datetime_inline') IS NOT NULL DROP FUNCTION dbo.fn_date_from_datetime_inline
go

CREATE FUNCTION dbo.fn_date_from_datetime_inline (
	@dt DATETIME
) RETURNS TABLE
AS RETURN SELECT CONVERT(DATE, @dt) TheDate
go
USE AdventureWorks2014
go

IF OBJECT_ID('tempdb..#temp_destination') IS NOT NULL DROP TABLE #temp_destination
go

SET STATISTICS TIME ON
go -->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


SELECT sod.*
	 , (SELECT theDate FROM tempdb.dbo.fn_date_from_datetime_inline(sod.ModifiedDate)) ModDate	 
	 INTO #temp_destination
	FROM Sales.SalesOrderDetail sod
	INNER JOIN Production.Product p ON sod.ProductID = p.ProductID
	ORDER BY Style
GO -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>



-- return cost threshold to 5
sp_configure 'cost threshold for parallelism', 5;
RECONFIGURE
go -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

---------------------- end of demo -----------------------------------------		