-- Demonstration script for the presentation "Improving SQL Server Performance by Writing Faster T-SQL" by Andy Novick
-- Copyright (C) 2012-2014 by Andrew Novick,
-- This script is intended for demonstration purposes only. Do not use this in production situations.

if exists (select * FROM sys.server_event_sessions WHERE name='count_functions')
               drop EVENT SESSION [count_functions] ON SERVER 
go

SELECT DB_ID('tempdb') [database id of tempdb]
go -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


CREATE EVENT SESSION [count_functions] ON SERVER 
ADD EVENT sqlserver.module_end(
     ACTION(sqlserver.database_id) -- database_id isn't one of the standard outputs of module_end so add it
    where sqlserver.database_id=2 and (object_type ='FN' or object_type= 'TF') )
ADD TARGET package0.asynchronous_bucketizer
(   SET filtering_event_name = 'sqlserver.module_end', source_type=0, source = 'object_id')
    WITH (EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS
         ,MAX_DISPATCH_LATENCY=30 SECONDS)
go -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>




select * from sys.dm_xe_sessions

ALTER EVENT SESSION count_functions on SERVER STATE=START
select * from sys.dm_xe_sessions
go


use tempdb
select DB_ID()

go
if OBJECT_ID('tempdb.dbo.fn_constant') IS NOT NULL DROP FUNCTION tempdb.dbo.fn_constant
GO
create function dbo.fn_constant () returns datetime as begin return '2013-05-01' end
go
if OBJECT_ID('tempdb.dbo.fn_date_from_datetime') IS NOT NULL DROP FUNCTION dbo.fn_date_from_datetime
GO
create function dbo.fn_date_from_datetime (@datetime datetime) returns datetime 
as begin return convert(datetime, convert(varchar(10), @datetime, 112)) end
go

select n.*, dbo.fn_constant() 
   from  AdventureWorks2014.Production.TransactionHistory n
   where dbo.fn_date_from_datetime(dateadd(minute, n.TransactionID, dbo.fn_constant())) < '2014-04-01'
   order by dbo.fn_date_from_datetime(dateadd(second, n.TransactionID, dbo.fn_constant()))
go



-- Take a look at the XML
SELECT CAST(target_data as XML) target_data
FROM sys.dm_xe_sessions AS s 
JOIN sys.dm_xe_session_targets t
    ON s.address = t.event_session_address
WHERE s.name = 'count_functions'
  AND t.target_name =  'histogram' -- This was for 2008R2 'asynchronous_bucketizer'
go


-- Easier to look at it as data
SELECT   n.value('(value)[1]', 'int') AS [object_id],
		 object_name(n.value('(value)[1]', 'int')) function_name,
         n.value('(@count)[1]', 'bigint') AS event_count
FROM (SELECT CAST(target_data as XML) target_data
		FROM sys.dm_xe_sessions AS s 
		JOIN sys.dm_xe_session_targets t
			ON s.address = t.event_session_address
		WHERE s.name = 'count_functions'
		  AND t.target_name = 'Histogram' /* 2008R2 'asynchronous_bucketizer'*/) as tab
	CROSS APPLY target_data.nodes('HistogramTarget/Slot'/*'BucketizerTarget/Slot'*/) as q(n)
go -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 

alter event session count_functions on server state=stop
go

if exists (select * FROM sys.server_event_sessions WHERE name='count_functions')
               drop EVENT SESSION [count_functions] ON SERVER 
go -- >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
