USE tempdb
GO
SET ANSI_NULLS ON
SET QUOTED_IDENTIFIER ON
SET ANSI_PADDING ON
GO
IF OBJECT_id('tempdb.dbo.trace_run') IS NOT NULL DROP TABLE tempdb.dbo.trace_run
go

CREATE TABLE [dbo].[trace_run](
	[trace_run_id] [int] IDENTITY(1,1) NOT NULL,
	[trace_project] [varchar](128) NULL,
	[trace_name] [varchar](128) NULL,
	[is_before] [bit] NOT NULL,
	[DB_ID] [int] NULL,
	[database_name] [sysname] NULL,
	[OBJECT_ID] [int] NULL,
	[schema_name] [sysname] NULL,
	[PROCEDURE_NAME] [sysname] NULL,
	[run_at_datetime] [datetime] NOT NULL,
	[elapsed_sec] [numeric](18, 3) NULL,
	[duration_sec] [numeric](18, 6) NULL,
	[cpu_sec] [numeric](18, 6) NULL,
	[logical_reads] [bigint] NULL,
	[logical_writes] [bigint] NULL,
	[physical_reads] [bigint] NULL,
	[proc_type] [varchar](10) NULL,
	[trace_text] [xml] NULL,
	[query_plan] [xml] NULL,
	[server_name] [sysname] NOT NULL,
	[login_id] [sysname] NOT NULL,
	[comment] [varchar](255) NULL,
	[current_datetime_forced] [bit] NOT NULL,
	[current_datetime] [datetime] NULL,
	[is_cache_warm] [bit] NULL,
	[cache_status] [varchar](16) NULL,
	PRIMARY KEY CLUSTERED (
	[trace_run_id] ASC
)) 

ALTER TABLE [dbo].[trace_run] ADD  CONSTRAINT [DF__trace_run__is_be__5FA9E18C]  DEFAULT ((0)) FOR [is_before]
ALTER TABLE [dbo].[trace_run] ADD  CONSTRAINT [DF__trace_run__run_a__609E05C5]  DEFAULT (getdate()) FOR [run_at_datetime]
ALTER TABLE [dbo].[trace_run] ADD  CONSTRAINT [DF__trace_run__serve__619229FE]  DEFAULT (@@servername) FOR [server_name]
ALTER TABLE [dbo].[trace_run] ADD  CONSTRAINT [DF__trace_run__login__62864E37]  DEFAULT (suser_sname()) FOR [login_id]
ALTER TABLE [dbo].[trace_run] ADD  CONSTRAINT [DF__trace_run__curre__637A7270]  DEFAULT ((0)) FOR [current_datetime_forced]
ALTER TABLE [dbo].[trace_run] ADD  CONSTRAINT [DF__trace_run__is_ca__646E96A9]  DEFAULT ((0)) FOR [is_cache_warm]
GO





IF OBJECT_ID('tempdb.dbo.ns_trace_call_tree') IS NOT NULL DROP TABLE tempdb.dbo.ns_trace_call_tree;
go

CREATE TABLE tempdb.[dbo].[ns_trace_call_tree](
	[trace_run_id] [int] NOT NULL,
	[time] [datetime] NOT NULL,
	[w] [varchar](3) NULL,
	[sec] [float] NULL,
	[dth] [varchar](3) NULL,
	[procedure] [nvarchar](776) NULL,
	[starttime] [datetime] NULL,
	[endtime] [datetime] NULL,
	[eventsequence] [int] NOT NULL,
	[d] [smallint] NULL
)
go

IF OBJECT_ID('tempdb.dbo.udf_Txt_SplitTAB') IS NOT NULL 
     AND db_id()=2 BEGIN; DROP FUNCTION dbo.udf_Txt_SplitTAB END;
go

CREATE FUNCTION [dbo].[udf_Txt_SplitTAB] (

    @sInputList varchar(8000) -- List of delimited items
  , @Delimiter char(1) = ',' -- delimiter that separates items
)   RETURNS @List TABLE (Item varchar(8000))
    WITH SCHEMABINDING
/* 
* Returns a table of strings that have been split by a delimiter.
* Similar to the Visual Basic (or VBA) SPLIT function. The 
* strings are trimmed before being returned.  Null items are not
* returned so if there are multiple separators between items, 
* only the non-null items are returned.
* Space is not a valid delimiter.
*
* Example:
select * FROM dbo.udf_Txt_SplitTAB('abcd,123, 456, efh,,hi', ',')
*
* Test:
DECLARE @Count int, @Delim char(10), @Input varchar(128)
SELECT @Count = Count(*) 
    FROM dbo.udf_Txt_SplitTAB('abcd,123, 456', ',')
PRINT 'TEST 1 3 lines:' + CASE WHEN @Count=3 
              THEN 'Worked' ELSE 'ERROR' END
SELECT @DELIM=CHAR(10)
     , @INPUT = 'Line 1' + @delim + 'line 2' + @Delim
SELECT @Count = Count(*) 
    FROM dbo.udf_Txt_SplitTAB(@Input, @Delim)
PRINT 'TEST 2  LF    :' + CASE WHEN @Count=2 
              THEN 'Worked' ELSE 'ERROR' END
*
*  Copyright 2003 Andrew Novick http://www.NovickSoftware.com
* You may use this function in any of your SQL Server databases
* including databases that you sell, so long as they contain 
* other unrelated database objects. You may not publish this 
* UDF either in print or electronically.
* Published in T-SQL UDF of the Week Newsletter Vol 1 #30
http://www.NovickSoftware.com/UDFofWeek/UDFofWeek.htm
***************************************************************/
AS BEGIN

DECLARE @Item Varchar(8000)
DECLARE @Pos int -- Current Starting Position
      , @NextPos int -- position of next delimiter
      , @LenInput int -- length of input
      , @LenNext int -- length of next item
      , @DelimLen int -- length of the delimiter

SELECT @Pos = 1
     , @DelimLen = LEN(@Delimiter) --  usually 1 
     , @LenInput = LEN(@sInputList)
     , @NextPos = CharIndex(@Delimiter, @sInputList, 1) 

-- Doesn't work for space as a delimiter
IF @Delimiter = ' ' BEGIN
   INSERT INTO @List 
       SELECT 'ERROR: Blank is not a valid delimiter'
   RETURN
END

-- loop over the input, until the last delimiter.
While @Pos <= @LenInput and @NextPos > 0 BEGIN

    IF @NextPos > @Pos BEGIN -- another delimiter found
       SET @LenNext = @NextPos - @Pos           
       Set @Item = LTrim(RTrim(
                            substring(@sInputList
                                   , @Pos
                                  , @LenNext)
                               )
                         ) 
       IF LEN(@Item) > 0 
           Insert Into @List Select @Item
       -- ENDIF

    END -- IF

    -- Position over the next item
    SELECT @Pos = @NextPos + @DelimLen
         , @NextPos = CharIndex(@Delimiter
                              , @sInputList
                              , @Pos) 
END

-- Now there might be one more item left
SET @Item = LTrim(RTrim(
                      SUBSTRING(@sInputList
                               , @Pos
                               , @LenInput-@Pos + 1)
                       )
                 )

IF Len(@Item) > 0 -- Put the last item in, if found
   INSERT INTO @List SELECT @Item

RETURN
END
GO

IF OBJECT_ID('tempdb.dbo.[ns_trace_sp_call_tree]') IS NOT NULL
	AND db_id()=2 DROP PROC dbo.[ns_trace_sp_call_tree]
go


CREATE proc [dbo].[ns_trace_sp_call_tree]

    @trace_name NVARCHAR(255),
    @directory nvarchar(255),
    @exclude_object_name_pipe_seperated_list NVARCHAR(MAX),
	@traceID int OUTPUT,
	@filename NVARCHAR(256) output
	

/* Scripts a trace including SP:Start and SP:Complete so that a call tree can be 
* created from it. 
*
* example
declare @traceID int, @fn sysname
exec  [dbo].[ns_trace_sp_call_tree] 'mytrace', 'e:\Traces\', 'foo|bar'
                                  , @traceid output, @fn output
select @traceID, @fn
exec sp_trace_setstatus, @traceId, 1
select * from sys.traces
exec sp_trace_setstatus, @traceId, 0
exec sp_trace_setstatus, @traceId, 2
*******************************************************************************/
as	

declare @rc int
declare @maxfilesize bigint
declare @dbaTraceID int
declare @DateTime datetime
DECLARE @StartDT  datetime = getdate()

set nocount on
set xact_abort on

SET @rc = 0;

-- trim the directory and ensure it ends with a \
SET @directory = LTRIM(RTRIM(@directory))
               + CASE WHEN RIGHT(RTRIM(@directory), 1) = '\' THEN '' ELSE '\' end

set @maxfilesize = 100 

set @FileName= @directory + 'sp_trace ' + @trace_name + ' ' + replace(CONVERT(varchar(19), getdate(), 121),':','-')

set @DateTime = DATEADD(day, 1, @StartDT)

exec @rc = sp_trace_create @TraceID OUTPUT
                         , 2 -- rollover files
                         , @FileName -- where to put it
                         , @maxfilesize --
                         , @DateTime    -- when to stop, we give it a day
if (@rc != 0) RETURN @rc

-- Set the events
declare @on bit
set @on = 1


-- Client side File and Table cannot be scripted

-- Set the events
exec sp_trace_setevent @TraceID, 10, 15, @on
exec sp_trace_setevent @TraceID, 10, 16, @on
exec sp_trace_setevent @TraceID, 10, 9, @on
exec sp_trace_setevent @TraceID, 10, 17, @on
exec sp_trace_setevent @TraceID, 10, 2, @on
exec sp_trace_setevent @TraceID, 10, 10, @on
exec sp_trace_setevent @TraceID, 10, 18, @on
exec sp_trace_setevent @TraceID, 10, 11, @on
exec sp_trace_setevent @TraceID, 10, 12, @on
exec sp_trace_setevent @TraceID, 10, 13, @on
exec sp_trace_setevent @TraceID, 10, 6, @on
exec sp_trace_setevent @TraceID, 10, 14, @on
exec sp_trace_setevent @TraceID, 43, 7, @on
exec sp_trace_setevent @TraceID, 43, 15, @on
exec sp_trace_setevent @TraceID, 43, 8, @on
exec sp_trace_setevent @TraceID, 43, 48, @on
exec sp_trace_setevent @TraceID, 43, 64, @on
exec sp_trace_setevent @TraceID, 43, 1, @on
exec sp_trace_setevent @TraceID, 43, 9, @on
exec sp_trace_setevent @TraceID, 43, 41, @on
exec sp_trace_setevent @TraceID, 43, 49, @on
exec sp_trace_setevent @TraceID, 43, 2, @on
exec sp_trace_setevent @TraceID, 43, 10, @on
exec sp_trace_setevent @TraceID, 43, 26, @on
exec sp_trace_setevent @TraceID, 43, 34, @on
exec sp_trace_setevent @TraceID, 43, 50, @on
exec sp_trace_setevent @TraceID, 43, 66, @on
exec sp_trace_setevent @TraceID, 43, 3, @on
exec sp_trace_setevent @TraceID, 43, 11, @on
exec sp_trace_setevent @TraceID, 43, 35, @on
exec sp_trace_setevent @TraceID, 43, 51, @on
exec sp_trace_setevent @TraceID, 43, 4, @on
exec sp_trace_setevent @TraceID, 43, 12, @on
exec sp_trace_setevent @TraceID, 43, 28, @on
exec sp_trace_setevent @TraceID, 43, 60, @on
exec sp_trace_setevent @TraceID, 43, 5, @on
exec sp_trace_setevent @TraceID, 43, 13, @on
exec sp_trace_setevent @TraceID, 43, 29, @on
exec sp_trace_setevent @TraceID, 43, 6, @on
exec sp_trace_setevent @TraceID, 43, 14, @on
exec sp_trace_setevent @TraceID, 43, 22, @on
exec sp_trace_setevent @TraceID, 43, 62, @on
exec sp_trace_setevent @TraceID, 42, 7, @on
exec sp_trace_setevent @TraceID, 42, 8, @on
exec sp_trace_setevent @TraceID, 42, 64, @on
exec sp_trace_setevent @TraceID, 42, 1, @on
exec sp_trace_setevent @TraceID, 42, 9, @on
exec sp_trace_setevent @TraceID, 42, 41, @on
exec sp_trace_setevent @TraceID, 42, 49, @on
exec sp_trace_setevent @TraceID, 42, 2, @on
exec sp_trace_setevent @TraceID, 42, 6, @on
exec sp_trace_setevent @TraceID, 42, 10, @on
exec sp_trace_setevent @TraceID, 42, 14, @on
exec sp_trace_setevent @TraceID, 42, 22, @on
exec sp_trace_setevent @TraceID, 42, 26, @on
exec sp_trace_setevent @TraceID, 42, 34, @on
exec sp_trace_setevent @TraceID, 42, 50, @on
exec sp_trace_setevent @TraceID, 42, 62, @on
exec sp_trace_setevent @TraceID, 42, 66, @on
exec sp_trace_setevent @TraceID, 42, 3, @on
exec sp_trace_setevent @TraceID, 42, 11, @on
exec sp_trace_setevent @TraceID, 42, 35, @on
exec sp_trace_setevent @TraceID, 42, 51, @on
exec sp_trace_setevent @TraceID, 42, 4, @on
exec sp_trace_setevent @TraceID, 42, 12, @on
exec sp_trace_setevent @TraceID, 42, 28, @on
exec sp_trace_setevent @TraceID, 42, 60, @on
exec sp_trace_setevent @TraceID, 42, 5, @on
exec sp_trace_setevent @TraceID, 42, 29, @on
exec sp_trace_setevent @TraceID, 12, 15, @on
exec sp_trace_setevent @TraceID, 12, 16, @on
exec sp_trace_setevent @TraceID, 12, 1, @on
exec sp_trace_setevent @TraceID, 12, 9, @on
exec sp_trace_setevent @TraceID, 12, 17, @on
exec sp_trace_setevent @TraceID, 12, 6, @on
exec sp_trace_setevent @TraceID, 12, 10, @on
exec sp_trace_setevent @TraceID, 12, 14, @on
exec sp_trace_setevent @TraceID, 12, 18, @on
exec sp_trace_setevent @TraceID, 12, 11, @on
exec sp_trace_setevent @TraceID, 12, 12, @on
exec sp_trace_setevent @TraceID, 12, 13, @on
exec sp_trace_setevent @TraceID, 13, 1, @on
exec sp_trace_setevent @TraceID, 13, 9, @on
exec sp_trace_setevent @TraceID, 13, 6, @on
exec sp_trace_setevent @TraceID, 13, 10, @on
exec sp_trace_setevent @TraceID, 13, 14, @on
exec sp_trace_setevent @TraceID, 13, 11, @on
exec sp_trace_setevent @TraceID, 13, 12, @on

-- Set the Filters
declare @intfilter int
declare @bigintfilter bigint

-- set a filter for this spid only
set @intfilter = @@SPID
exec sp_trace_setfilter @TraceID, 12, 0, 0, @intfilter


-- Parse the list and create a filter for each entry to exclude it
DECLARE @pos INT
      , @nextPos INT
      , @lenInput INT
      , @lenNext INT
      , @object_name NVARCHAR(776)
SELECT @pos = 1
     , @lenInput = LEN (@exclude_object_name_pipe_seperated_list)
     , @nextPos = CharIndex('|', @exclude_object_name_pipe_seperated_list, 1)
 
WHILE @pos <= @lenInput AND @nextPos > 0 BEGIN
	SET @lenNext = @nextPos - @pos 
	SET @object_name = LTRIM(RTRIM(SUBSTRING(@exclude_object_name_pipe_seperated_list, @pos, @lenNext)))
	
	exec sp_trace_setfilter @TraceID, 34, 0, 7, @object_name

	SELECT @pos = @nextPos + 1
	     , @nextPos = CHARINDEX('|',@exclude_object_name_pipe_seperated_list, @pos)
END
-- catch the last object
SET @object_name=LTrim(RTrim(substring(@exclude_object_name_pipe_seperated_list
                   , @Pos, @LenInput-@Pos + 1)))
IF LEN(@object_name) > 0 	exec sp_trace_setfilter @TraceID, 34, 0, 7, @object_name

-- DON'T Set the trace status to start
--exec sp_trace_setstatus @TraceID, 1

RETURN @rc

GO



IF OBJECT_ID('tempdb.dbo.ns_trace_call_tree_from_trace_table') IS NOT NULL
	AND db_id()=2 DROP PROC dbo.ns_trace_call_tree_from_trace_table
go

CREATE PROC [dbo].[ns_trace_call_tree_from_trace_table] 

	@trace_table_name sysname
  , @trace_run_id int
  , @exclusion_pipe_sep_list varchar(8000) = ''
  
AS

declare @num_exclusions int = 0

create table #exclusions (exclude_name sysname primary key clustered)

insert into #exclusions (exclude_name)
select Item from dbo.udf_Txt_SplitTAB(@Exclusion_pipe_sep_list, '|')
set @num_exclusions = @@ROWCOUNT

DECLARE @sql NVARCHAR(MAX)

SET @sql = '
WITH x AS (
SELECT c, w, objectname , starttime, endtime, CONVERT(NUMERIC(9,3), sec) Sec
     , EventSequence 
     , CASE WHEN W=''S'' THEN 1 ELSE -1 END D  
     , ROW_NUMBER() OVER (ORDER BY eventsequence) rn
FROM (
SELECT ''S'' W 
    , S.starttime c
    , e.objectname
    , e.starttime
    , e.endtime
    , e.duration/1000000.0 Sec
    , s.EventSequence
FROM 
(SELECT ''E'' W, t.*
	FROM '+ @trace_table_name +' t
	WHERE t.eventclass=43  AND  ObjectType not in (20038 /*scalar*/, 17985/*Agg*/, 18004 /* clr TVF*/)	 ) E
LEFT OUTER join
(SELECT ''S'' W, *
	FROM '+ @trace_table_name +' t
	WHERE t.eventclass=42 AND  ObjectType not in (20038 /*scalar*/, 17985/*Agg*/, 18004 /* clr TVF*/)	
	) S
ON S.starttime = e.starttime AND s.objectname=e.objectname

UNION ALL

SELECT  ''E'' w 
    , e.endtime C 
	, e.objectname
	, e.starttime
	, e.endtime
	, e.duration/1000000.0 Sec
	, e.eventSequence
	FROM '+ @trace_table_name +' E
	WHERE E.eventclass=43  AND  ObjectType not in (20038 /*scalar*/, 17985/*Agg*/, 18004 /* clr TVF*/)	
)Q	
' + case when @num_exclusions = 0 then '' else 'WHERE objectname NOT IN (select exclude_name from #exclusions)' END +'
)
insert into dbo.ns_trace_call_tree (trace_run_id, time, w, sec, dth
                                 , [procedure],starttime,endtime,d,eventsequence)
SELECT ' + CONVERT(VARCHAR(30), @trace_run_id) + '
    , c [time]
    , w 
    , sec --RIGHT(SPACE(20) + CONVERT(VARCHAR(30),sec), 12) sec
    , COALESCE((SELECT sum (d) FROM x x2 WHERE x2.rn < x.rn) + CASE WHEN w=''E'' THEN -1 ELSE 0 END, 0) [dth]
    , replicate(''  '', COALESCE((SELECT sum (d) FROM x x2 WHERE x2.rn < x.rn) + CASE WHEN w=''E'' THEN -1 ELSE 0 END, 0)) + oBJECTNAME  [procedure]
    , starttime
    , endtime
    , d
    , eventsequence
	FROM x
ORDER BY /*c
  , CASE WHEN w = ''S'' THEN 1 ELSE 2 END
  , */ eventsequence

'  
EXEC (@sql)
GO


IF OBJECT_ID('tempdb.dbo.trace_run_record_stats') IS NOT NULL 
	AND DB_ID() = 2 DROP PROC dbo.trace_run_record_stats
go

CREATE PROC [dbo].[trace_run_record_stats] 

	@trace_procedure VARCHAR(128)
  , @trace_name VARCHAR(128)
  , @start_datetime DATETIME
  , @cache_status VARCHAR(16)
  , @trace_run_id INT = null OUTPUT
  
/* gathers procedure and query statistics from the most recently run test query.
* This assumes that the tests are the only query on the system when run.
*/
as	

declare @top_object TABLE (trace_run_id int, database_id int NOT NULL, OBJECT_ID int NOT NULL)
INSERT INTO [dbo].[trace_run]
           ([trace_project]
           ,[trace_name]
           ,[db_id]
           ,[database_name]
           ,[object_id]
           ,[schema_name]
           ,[PROCEDURE_NAME]
           ,[run_at_datetime]
           ,elapsed_sec
           ,[duration_sec]
           ,[cpu_sec]
           ,[logical_reads]
           ,[logical_writes]
           ,[physical_reads]
           ,[proc_type]
           ,[trace_text]
           ,[query_plan]
           ,[server_name]
           ,[login_id]
           ,cache_status)
       OUTPUT INSERTED.trace_run_id
            , inserted.db_id
            , inserted.object_id    
            INTO @top_object (trace_run_id, database_id, object_id)
SELECT TOP 1
    @trace_procedure AS trace_project
  , @trace_name AS trace_name 
  , DB_ID() [DB_ID]
  , CASE WHEN DB_ID() = 32767 THEN 'Resource' ELSE DB_NAME(DB_ID()) end database_name
  , -1 object_id
  , NULL  AS [SCHEMA_NAME]  
  , NULL  proc_name
  , @start_datetime run_at_datetime
  , CONVERT(NUMERIC(18,3), CONVERT(FLOAT, DATEDIFF(Ms, @start_datetime, GETDATE()))/1000.0)
  , null duration_sec
  , null cpu_sec
  , NULL last_logical_reads
  , NULL last_logical_writes
  , NULL last_physical_reads
  , 'P' proc_type
  , NULL  trace_text
  , NULL query_plan
  , @@SERVERNAME server_name
  , SUSER_SNAME() login_id
  , @cache_status

DECLARE @db_id INT, @object_id INT
SELECT TOP 1 @trace_run_id = trace_run_id, @db_id=database_id, @object_id=OBJECT_ID FROM @top_object;

GO


IF OBJECT_ID('tempdb.dbo.ns_trace_call_tree_for_proc') IS NOT NULL 
	AND db_id()=2 DROP PROC dbo.ns_trace_call_tree_for_proc
go

CREATE PROCEDURE [dbo].[ns_trace_call_tree_for_proc]
	@sql [nvarchar](max),
	@procedure_name [sysname] = NULL,
	@trace_name [nvarchar](255),
	@cache_status [varchar](16) = NULL,
	@directory [nvarchar](255),
	@exclude_object_name_pipe_seperated_list [nvarchar](max),
	@min_sub_proc_time_sec real = 0.015,
	@min_body_time_sec real = 0.025,
	@trace_run_id [int] = NULL OUTPUT
WITH EXECUTE AS CALLER
AS
declare @rc int
declare @maxfilesize bigint
declare @TraceID int
declare @DateTime datetime
DECLARE @StartDT  DATETIME2(7)
DECLARE @filename nvarchar(255)

set nocount on
set xact_abort on

SET @rc = 0;

-- Create the trace
exec  [dbo].[ns_trace_sp_call_tree] @trace_name, @directory, @exclude_object_name_pipe_seperated_list
                                  , @traceid output, @filename OUTPUT
                                  
-- start the trace
exec sp_trace_setstatus @traceId, 1

SET @StartDT = SYSDATETIME()

EXEC (@sql) -- EXECUTE the statement

-- stop and delete the trace
exec sp_trace_setstatus @traceId, 0
exec sp_trace_setstatus @traceId, 2

EXEC [dbo].[trace_run_record_stats] 
   @trace_procedure = @procedure_name
  ,@trace_name      = @trace_name
  ,@start_datetime  = @startDT
  ,@cache_status    = @cache_status
  ,@trace_run_id    = @Trace_run_id OUTPUT

-- add the .trc extenstion to the trace filename
SET @filename = @filename + '.trc'

-- get rid of the temp table if it exists
IF OBJECT_ID('tempdb..#MyTraceTable', 'U') IS NOT NULL DROP TABLE #MyTraceTable

select *  into #MyTraceTable
from ::fn_trace_gettable (@filename, 999)

-- calculate the call tree
EXEC [ns_trace_call_tree_from_trace_table] 
           '#MyTraceTable'
         , @trace_run_id
         , @exclude_object_name_pipe_seperated_list

--SELECT [time]
--    , w 
--    , RIGHT(SPACE(20) + CONVERT(VARCHAR(30),sec), 12) sec
--    , [dth]
--    , [procedure]
--    --, starttime
--    --, endtime
--    --, d
--	FROM dbo.ns_trace_call_tree
--	WHERE trace_run_id = @trace_run_id
--	ORDER BY eventsequence

;WITH e AS (
	SELECT [time], w, sec, CONVERT(DECIMAL(18,1), dth) dth
	      , [procedure], starttime, endtime
	      , CONVERT(BIGINT,eventsequence)*100 eventsequence 
		  , ROW_NUMBER() OVER (ORDER BY eventsequence) rn
			FROM dbo.ns_trace_call_tree
		WHERE trace_run_id = @trace_run_id
			--AND dth<2
     )
, singleton as (
SELECT e.[time]
     ,'C' W, e.sec
     , /*CASE WHEN ISNUMERIC(e.dth)=1 THEN CONVERT(DECIMAL(18,1), e.dth) ELSE CONVERT(DECIMAL(18,1), 0) end*/ e.dth
     , e.[procedure]
     , e.starttime
     , n.endtime
     , n.eventsequence
     , e.rn
     , DATEDIFF (millisecond, e.time, n.time) diff_ms
     , n.rn rnnext
     , n.[time] timenext
	FROM e
		LEFT OUTER JOIN e n ON e.rn = n.rn-1
	WHERE e.W='S' AND n.W = 'E' AND e.dth = n.dth AND e.[procedure] = n.[procedure]	
)
, h AS (
SELECT e.[time]
      , CASE WHEN e.W='E' AND p.W='S' AND e.dth = p.dth	 AND e.[procedure] = p.[PROCEDURE]
             THEN 'C'
             ELSE e.W END   w 
      , e.sec
      , e.dth
      , e.[procedure]/*, e.starttime, e.endtime*/
      , e.eventsequence
      , e.rn
      , DATEDIFF (millisecond, e.time, n.time) n_diff_ms
      , DATEDIFF (millisecond, e.time, p.time) p_diff_ms
      , n.rn rnNEXT
      , n.[time] timenext
      , n.w wnext
      , n.sec secnext
      , n.dth dthnext
      , n.[procedure] procnext
      , n.eventsequence eventseqnext 
      , p.rn rnprev
      , p.[time] timeprev
      , p.w wprev
      , p.sec secprev
      , p.dth dthprev
      , p.[procedure] procprev
      , p.eventsequence eventseqprev
      , e.starttime
      , e.endtime
	FROM e
		LEFT OUTER JOIN e n ON e.rn = n.rn-1
		LEFT OUTER JOIN e p ON e.rn = p.rn + 1
	WHERE NOT (e.W='S' AND n.W = 'E' AND e.dth = n.dth AND e.[procedure] = n.[procedure])
	  AND (1=CASE when e.W='E' AND p.W='S' AND e.dth = p.dth	 AND e.[procedure] = p.[PROCEDURE]	
	              THEN CASE WHEN e.sec >= @min_sub_proc_time_sec THEN 1 ELSE 0 END
	              ELSE 1
	              END)
)
, pbody AS (
	SELECT DATEADD (millisecond, DATEDIFF(millisecond, TIMEprev, [time])/2,TIMEprev) TIME
	     , 'B' W 
	     , CONVERT(DECIMAL(18,3), DATEDIFF (millisecond, TIMEprev, [TIME])/1000.0) sec
	     , dthprev - 0.5 dth
	     , SPACE(dthprev*2) + '<body of ' + RTRIM(LTRIM([procedure])) + '>' [procedure]
	     , (eventsequence - 10) eventsequence
	     , h.starttime, h.endtime
		FROM h
		WHERE w=wprev AND dth = dthprev - 1 
		--ORDER BY eventsequence
)
, nbody AS (
	SELECT DATEADD (millisecond, DATEDIFF(millisecond, [time],TIMEnext)/2,TIME) TIME
	     , 'B' W 
	     , CONVERT(DECIMAL(18,3), DATEDIFF (millisecond, [TIME], TIMEnext)/1000.0) sec
	     , dth + 0.5 dth
	     , SPACE(dthnext*2) + '<body of ' + RTRIM(LTRIM([procedure])) + '>' [procedure]
	     , (eventsequence + 10) eventsequence
	     , h.starttime, h.endtime
	     
		FROM h
		WHERE w=wnext AND dth = dthnext - 1 
		--ORDER BY eventsequence
)
SELECT TIME
     , W 
     , sec
     , dth
     , [procedure]
     , eventsequence
     --, starttime
     --, endtime
   FROM h
UNION ALL      
SELECT TIME
     , W 
     , sec
     , dth
     , [procedure]
     , eventsequence
     --, starttime
     --, endtime
	FROM pbody
	WHERE sec> @min_body_time_sec
UNION ALL      
SELECT TIME
     , W 
     , sec
     , dth
     , [procedure]
     , eventsequence
     --, starttime
     --, endtime
	FROM nbody
	WHERE sec> @min_body_time_sec
ORDER BY eventsequence     	

RETURN @rc

GO